@@ -1,10 +1,13 @@
 import argparse
+from pathlib import Path
+from typing import Optional
 
 from mohou.dataset import AutoRegressiveDatasetConfig
 from mohou.default import (
     create_default_encoding_rule,
     create_default_image_context_list,
 )
+from mohou.file import get_project_path
 from mohou.model.lstm import LSTMConfig
 from mohou.script_utils import create_default_logger, train_lstm
 from mohou.setting import setting
@@ -13,6 +16,7 @@ from mohou.trainer import TrainConfig
 if __name__ == "__main__":
     parser = argparse.ArgumentParser()
     parser.add_argument("-pn", type=str, default=setting.primary_project_name, help="project name")
+    parser.add_argument("-pp", type=str, help="project path. preferred over pn.")
     parser.add_argument("-n", type=int, default=3000, help="iteration number")
     parser.add_argument("-aug", type=int, default=2, help="number of augmentation X")
     parser.add_argument("-hidden", type=int, default=200, help="number of hidden state of lstm")
@@ -28,19 +32,25 @@ if __name__ == "__main__":
     )
     args = parser.parse_args()
 
-    project_name = args.pn
-    n_epoch = args.n
-    n_aug = args.aug
-    n_hidden = args.hidden
-    n_layer = args.layer
-    cov_scale = args.cov_scale
-    valid_ratio = args.valid_ratio
-    warm_start = args.warm
-    use_image_context = args.use_image_context
+    project_name: str = args.pn
+    project_path_str: Optional[str] = args.pp
+    n_epoch: int = args.n
+    n_aug: int = args.aug
+    n_hidden: int = args.hidden
+    n_layer: int = args.layer
+    cov_scale: float = args.cov_scale
+    valid_ratio: float = args.valid_ratio
+    warm_start: bool = args.warm
+    use_image_context: bool = args.use_image_context
 
-    logger = create_default_logger(project_name, "lstm")  # noqa
+    if project_path_str is None:
+        project_path = get_project_path(project_name)
+    else:
+        project_path = Path(project_path_str)
 
-    encoding_rule = create_default_encoding_rule(project_name)
+    logger = create_default_logger(project_path, "lstm")  # noqa
+
+    encoding_rule = create_default_encoding_rule(project_path)
     model_config = LSTMConfig(
         encoding_rule.dimension,
         n_hidden=n_hidden,
@@ -53,11 +63,11 @@ if __name__ == "__main__":
 
     context_list = None
     if use_image_context:
-        context_list = create_default_image_context_list(project_name)
+        context_list = create_default_image_context_list(project_path)
         model_config.n_static_context = len(context_list[0])
 
     train_lstm(
-        project_name,
+        project_path,
         encoding_rule,
         model_config,
         dataset_config,
