@@ -3,10 +3,11 @@ import random
 import time
 from logging import Logger
 from pathlib import Path
-from typing import Dict, List, Optional, Type
+from typing import Any, Dict, List, Optional, Tuple, Type
 
 import matplotlib.pyplot as plt
 import numpy as np
+import tqdm
 
 from mohou.model.autoencoder import VariationalAutoEncoder
 
@@ -37,12 +38,12 @@ from mohou.model.lstm import LSTMBase, LSTMConfigBase
 from mohou.propagator import PropagatorBase
 from mohou.trainer import TrainCache, TrainConfig, train
 from mohou.types import (
-    AngleVector,
     ElementDict,
     EpisodeBundle,
-    GripperState,
+    EpisodeData,
     ImageBase,
     TerminateFlag,
+    VectorBase,
 )
 from mohou.utils import canvas_to_ndarray
 
@@ -240,115 +241,182 @@ def add_text_to_image(image: ImageBase, text: str, color: str):
     ax = plt.subplot(1, 1, 1)
     ax.axis("off")
     ax.imshow(image.to_rgb()._data)
-    bbox = dict(boxstyle="round", facecolor="white", alpha=0.7)
+    bbox = dict(boxstyle="round", facecolor="white", alpha=0.4)
     ax.text(7, 1, text, fontsize=15, color=color, verticalalignment="top", bbox=bbox)
     fig.canvas.draw()
     fig.canvas.flush_events()
     return canvas_to_ndarray(fig)
 
 
+class VectorSequencePlotter:
+    fig: Any
+    axs: List
+    axis_index_table: Dict[Tuple[Type[VectorBase], int], int]
+    inverse_axis_index_table: List[Tuple[Type[VectorBase], int]]
+    min_val_per_axis: List[float]
+    max_val_per_axis: List[float]
+    is_finalized: bool = False
+
+    def __init__(self, project_path: Path, plot_elem_types: List[Type[VectorBase]]) -> None:
+        bundle = EpisodeBundle.load(project_path)
+
+        axis_index_table: Dict[Tuple[Type[VectorBase], int], int] = {}
+        inverse_axis_index_table: List[Tuple[Type[VectorBase], int]] = []
+        axis_count = 0
+        for elem_type in plot_elem_types:
+            shape = bundle.type_shape_table[elem_type]
+            assert len(shape) == 1  # because it's a vector
+            for i in range(shape[0]):
+                axis_index_table[(elem_type, i)] = axis_count
+                inverse_axis_index_table.append((elem_type, i))
+                axis_count += 1
+
+        fig, axs = plt.subplots(axis_count, 1)
+        if axis_count == 1:
+            axs = [axs]
+
+        for ax in axs:
+            ax.set_xticklabels([])
+            ax.yaxis.set_tick_params(labelsize=5)
+
+        fig.tight_layout(pad=3.0)
+
+        self.fig = fig
+        self.axs = axs
+        self.axis_index_table = axis_index_table
+        self.inverse_axis_index_table = inverse_axis_index_table
+        self.min_val_per_axis = [np.inf for _ in range(axis_count)]
+        self.max_val_per_axis = [-np.inf for _ in range(axis_count)]
+
+    def add_plot(
+        self,
+        x_seq: Optional[np.ndarray],
+        y_seq: np.ndarray,
+        elem_type: Type[VectorBase],
+        idx: int,
+        **kwargs
+    ):
+        assert y_seq.ndim == 1
+        assert not self.is_finalized
+        axis_index = self.axis_index_table[(elem_type, idx)]
+        if x_seq is not None:
+            assert x_seq.ndim == 1
+            self.axs[axis_index].plot(x_seq, y_seq, **kwargs)
+        else:
+            self.axs[axis_index].plot(y_seq, **kwargs)
+        self.min_val_per_axis[axis_index] = min(self.min_val_per_axis[axis_index], np.min(y_seq))
+        self.max_val_per_axis[axis_index] = max(self.max_val_per_axis[axis_index], np.max(y_seq))
+
+    def finalize(self):
+        ax_last = self.axs[-1]
+        ax_last.legend(fontsize=5)
+
+        for axis_index, ax in enumerate(self.axs):
+            val_min = self.min_val_per_axis[axis_index]
+            val_max = self.max_val_per_axis[axis_index]
+            margin = max(0.1, 0.1 * (val_max - val_min))
+            ax.set_ylim(val_min - margin, val_max + margin)
+            ax.grid()
+
+            elem_type, vector_index = self.inverse_axis_index_table[axis_index]
+            ax.text(
+                0.5,
+                0.5,
+                "{} dimension {}".format(elem_type.__name__, vector_index),
+                fontsize=5,
+                transform=ax.transAxes,
+                horizontalalignment="center",
+                verticalalignment="center",
+                bbox=dict(boxstyle="round", facecolor="white", alpha=0.7),
+            )
+        self.is_finalized = True
+
+    def save_fig(self, image_path: Path):
+        assert self.is_finalized
+        self.fig.savefig(str(image_path), format="png", dpi=300)
+
+
 def visualize_lstm_propagation(project_path: Path, propagator: PropagatorBase, n_prop: int):
     bundle = EpisodeBundle.load(project_path).get_untouch_bundle()
     save_dir_path = project_path / "lstm_result"
     save_dir_path.mkdir(exist_ok=True)
     prop_name = propagator.__class__.__name__
 
+    if propagator.require_static_context:
+        raise NotImplementedError("not implemented yet")
+
     image_encoder = load_default_image_encoder(project_path)
+    image_type = image_encoder.elem_type
 
-    for idx, edata in enumerate(bundle):
-        episode_data = bundle[idx]
-
-        image_type = None
-        for key, encoder in propagator.encoding_rule.items():
-            if issubclass(key, ImageBase):
-                image_type = key
-        assert image_type is not None
-
-        n_feed = 20
-        feed_avs = episode_data.get_sequence_by_type(AngleVector)[:n_feed]
-        feed_images = episode_data.get_sequence_by_type(image_type)[:n_feed]
-
-        # set context if necessary
-        if propagator.require_static_context:
-            context = image_encoder.forward(feed_images[0])
-            propagator.set_static_context(context)
-
-        use_gripper = GripperState in propagator.encoding_rule
-
-        if use_gripper:
-            fed_grippers = episode_data.get_sequence_by_type(GripperState)[:n_feed]
+    for idx, episode_data in enumerate(bundle):
 
         print("start lstm propagation")
+        n_feed = 20
+        feed_partial_episode = episode_data[:n_feed]
         for i in range(n_feed):
-            elem_dict = ElementDict([feed_avs[i], feed_images[i]])
-            if use_gripper:
-                elem_dict[GripperState] = fed_grippers[i]
-            propagator.feed(elem_dict)
+            feed_edict = feed_partial_episode[i]
+            propagator.feed(feed_edict)
+        pred_edict_list = propagator.predict(n_prop)
+        pred_partial_episode = EpisodeData.from_edict_list(
+            pred_edict_list, check_terminate_flag=False
+        )
         print("finish lstm propagation")
 
-        elem_dict_list = propagator.predict(n_prop)
-        pred_images = [elem_dict[image_type] for elem_dict in elem_dict_list]
-        pred_flags = [elem_dict[TerminateFlag].numpy().item() for elem_dict in elem_dict_list]
-        pred_avs = [elem_dict[AngleVector].numpy() for elem_dict in elem_dict_list]
-        if use_gripper:
-            pred_gss = [elem_dict[GripperState].numpy() for elem_dict in elem_dict_list]
+        # Plot history of Vector types
+        elem_types = propagator.encoding_rule.keys()
+        vector_elem_types = [et for et in elem_types if issubclass(et, VectorBase)]
 
-        n_av_dim = bundle.spec.type_shape_table[AngleVector][0]
-        n_gs_dim = bundle.spec.type_shape_table[GripperState][0] if use_gripper else 0
-        fig, axs = plt.subplots(n_av_dim + n_gs_dim, 1)
+        vecseq_plotter = VectorSequencePlotter(project_path, vector_elem_types)
 
-        # plot angle vectors
-        av_seq_gt = episode_data.get_sequence_by_type(AngleVector)
-        np_av_seq_gt = np.array([av.numpy() for av in av_seq_gt])
-        np_av_seq_pred = np.concatenate((np_av_seq_gt[:n_feed], np.array(pred_avs)), axis=0)
+        for elem_type in vector_elem_types:
 
-        i_dim = 0
-        for i_av_dim in range(n_av_dim):
-            axs[i_dim].plot(np_av_seq_gt[:, i_av_dim], color="blue", lw=1)
-            axs[i_dim].plot(np_av_seq_pred[:, i_av_dim], color="red", lw=1)
+            # create ground truth data
+            seq_gt = episode_data.get_sequence_by_type(elem_type)
+            np_seq_gt = np.array([e.numpy() for e in seq_gt])
 
-            # determine axes min max
-            conc = np.hstack((np_av_seq_gt[:, i_av_dim], np_av_seq_pred[:, i_av_dim]))
-            y_min = np.min(conc)
-            y_max = np.max(conc)
-            diff = y_max - y_min
-            axs[i_dim].set_ylim([y_min - diff * 0.1, y_max + diff * 0.1])
-            axs[i_dim].set_title("AngleVector dim {}".format(i_av_dim), fontsize=5, pad=0.0)
-            i_dim += 1
+            # create feed + pred data
+            seq_feed = feed_partial_episode.get_sequence_by_type(elem_type)
+            seq_pred = pred_partial_episode.get_sequence_by_type(elem_type)
+            np_seq_feedpred = np.array(
+                [e.numpy() for e in seq_feed] + [e.numpy() for e in seq_pred]
+            )
 
-        if use_gripper:
-            gs_seq_gt = episode_data.get_sequence_by_type(GripperState)
-            np_gs_seq_gt = np.array([gs.numpy() for gs in gs_seq_gt])
-            np_gs_seq_pred = np.concatenate((np_gs_seq_gt[:n_feed], np.array(pred_gss)), axis=0)
-            for i_gs_dim in range(n_gs_dim):
-                axs[i_dim].plot(np_gs_seq_gt[:, i_gs_dim], color="blue", lw=1)
-                axs[i_dim].plot(np_gs_seq_pred[:, i_gs_dim], color="red", lw=1)
+            n_dim = seq_feed.elem_shape[0]
+            for i in range(n_dim):
+                np_seq_gt_slice = np_seq_gt[:, i]
+                np_seq_feedpred_slice = np_seq_feedpred[:, i]
+                vecseq_plotter.add_plot(
+                    None, np_seq_gt_slice, elem_type, i, color="blue", lw=1, label="ground truth"
+                )
+                vecseq_plotter.add_plot(
+                    None,
+                    np_seq_feedpred_slice,
+                    elem_type,
+                    i,
+                    color="red",
+                    lw=1,
+                    label="feed + pred",
+                )
 
-                # determine axes min max
-                conc = np.hstack((np_gs_seq_gt[:, i_gs_dim], np_gs_seq_pred[:, i_gs_dim]))
-                y_min = np.min(conc)
-                y_max = np.max(conc)
-                diff = y_max - y_min
-                axs[i_dim].set_ylim([y_min - diff * 0.1, y_max + diff * 0.1])
-                axs[i_dim].set_title("GripperState dim {}".format(i_gs_dim), fontsize=5, pad=0.0)
-                i_dim += 1
-
-        for ax in axs:
-            ax.grid()
-
-        image_path = save_dir_path / "seq-{}-{}{}.png".format(prop_name, AngleVector.__name__, idx)
-        fig.savefig(str(image_path), format="png", dpi=300)
+        image_path = save_dir_path / "seq-vectors-{}{}.png".format(prop_name, idx)
+        vecseq_plotter.finalize()
+        vecseq_plotter.save_fig(image_path)
         print("saved to {}".format(image_path))
 
         # save gif image
         print("adding text to images...")
+        feed_images = feed_partial_episode.get_sequence_by_type(image_type)
         fed_images_with_text = [
             add_text_to_image(image, "fed (original) image)", "blue") for image in feed_images
         ]
         clamp = lambda x: max(min(x, 1.0), 0.0)  # noqa
+        pred_images = pred_partial_episode.get_sequence_by_type(image_type)
+        pred_flags = pred_partial_episode.get_sequence_by_type(TerminateFlag)
         pred_images_with_text = [
             add_text_to_image(
-                image, "predicted image (prob-terminated={:.2f})".format(clamp(flag)), "green"
+                image,
+                "predicted image (prob-terminated={:.2f})".format(clamp(flag.numpy().item())),
+                "green",
             )
             for image, flag in zip(pred_images, pred_flags)
         ]
@@ -359,3 +427,50 @@ def visualize_lstm_propagation(project_path: Path, propagator: PropagatorBase, n
         assert ImageSequenceClip is not None, "check if your moviepy is properly installed"
         clip = ImageSequenceClip(images_with_text, fps=20)
         clip.write_gif(str(image_path), fps=20)
+
+
+def plot_execution_result(
+    project_path: Path, propagator: PropagatorBase, edict_seq: List[ElementDict], n_prop: int = 10
+):
+    timestr = "_" + time.strftime("%Y%m%d%H%M%S")
+    propagator.clear_fed_data()
+
+    elem_types = propagator.encoding_rule.keys()
+    vector_elem_types = [et for et in elem_types if issubclass(et, VectorBase)]
+    vecseq_plotter = VectorSequencePlotter(project_path, vector_elem_types)
+
+    # plot actual data
+    episode = EpisodeData.from_edict_list(edict_seq, check_terminate_flag=False)
+    for elem_type in vector_elem_types:
+        seq = episode.get_sequence_by_type(elem_type)
+        np_seq = np.array([e.numpy() for e in seq])
+        n_dim = np_seq.shape[1]
+        for i in range(n_dim):
+            y_seq = np_seq[:, i]
+            vecseq_plotter.add_plot(None, y_seq, elem_type, i, color="blue", lw=1.0)
+
+    # plot prediction
+    for time_index, edict in enumerate(tqdm.tqdm(edict_seq)):
+        x_seq = np.array([time_index + 1 + j for j in range(n_prop)])
+
+        propagator.feed(edict)
+        edict_list_predicted = propagator.predict(n_prop)
+        episode_predicted = EpisodeData.from_edict_list(
+            edict_list_predicted, check_terminate_flag=False
+        )
+
+        for elem_type in vector_elem_types:
+            seq = episode_predicted.get_sequence_by_type(elem_type)
+            np_seq = np.array([e.numpy() for e in seq])
+            n_dim = np_seq.shape[1]
+
+            for i in range(n_dim):
+                y_seq = np_seq[:, i]
+                vecseq_plotter.add_plot(x_seq, y_seq, elem_type, i, color="red", lw=0.2)
+
+    save_dir_path = project_path / "execution_result"
+    save_dir_path.mkdir(exist_ok=True)
+    prop_name = propagator.__class__.__name__
+    image_path = save_dir_path / "result-{}-{}.png".format(prop_name, timestr)
+    vecseq_plotter.finalize()
+    vecseq_plotter.save_fig(image_path)
